#include "Cgiu.h"
#include "keypressed.h"
#include <stdio.h>

#ifdef linux 
#define SERIALDEVICE "/dev/ttyS0" // linux
#else 
#define SERIALDEVICE "/dev/ttya"  // solaris
#endif

Cgiu myCgiu;

// global variable force (the 5 finger forces)
float force[5] = {0.0, 0.0, 0.0, 0.0, 0.0};
   
void pos(int x,int y) {
   printf("\033[%i;%if",y,x);
}

void cls() {
   printf("\033[2J");
}

void updateconsole() {
   pos(1,10);
   printf("force[0] = %f\n", force[0]);
   printf("force[1] = %f\n", force[1]);
   printf("force[2] = %f\n", force[2]);
   printf("force[3] = %f\n", force[3]);
   printf("force[4] = %f\n\n", force[4]);
   printf("use keys to change force values!\n");
   printf("to exit program press ESC.\n");
}


int readAndDebug(int fd, int num) {
   printf("reading from device...", num);
   int check = myCgiu.ReadData(fd, num);
   printf(" %d byte(s) read", check);
   if(check > 0) {
      printf(":\n");force[0]=0.0;

      for (int i = 0; i < check; i++) {
         printf("%d ", myCgiu.readBuffer[i]);
      }
   }
   printf("\n\n");
   return check;
}

void convertForces(float* forces, unsigned char* result)  {
   int intForceVal;
   for (int i = 0; i < 5; i++) {
      intForceVal = (int) (32767 * forces[i]);
      result[2*i] = (char)(intForceVal & 0x00FF);
      result[(2*i)+1] = (char)((intForceVal >> 8) & 0x00FF);
   }
}

void main(int argc, char *argv[]) {   
   char cc; // for keypressed function
   cls();
   pos(1,1);
   printf("ForceManager\n\n");
   printf("opening serial device...\n");
   int fd = myCgiu.Init(SERIALDEVICE);
   if (fd != -1) printf("OK.\n\n"); else exit(-1);   
   printf("Reset CyberGrasp ([^R])...\n");
   char RESET[1] = {18};
   write(fd, RESET, 1);
   sleep(4);
   readAndDebug(fd, 255);

   unsigned char force_converted[10];
   tty_noncanonical(STDIN_FILENO); // f. Tastatureingaben
 
   do {
      updateconsole();
      convertForces(force, force_converted);
      write(fd, "X", 1);
      write(fd, force_converted, 10);
      usleep(50000);
      if (keypressed(cc) == 1) {
         switch (cc) {
            case '1': { force[0] =  1.0; break; }
            case 'q': { force[0] = -1.0; break; }
            case 'a': { if (force[0] <  1.0) force[0] = force[0]+0.1; break; }
            case 'z': { if (force[0] > -1.0) force[0] = force[0]-0.1; break; }

            case '2': { force[1] =  1.0; break; }
            case 'w': { force[1] = -1.0; break; }
            case 's': { if (force[1] <  1.0) force[1] = force[1]+0.1; break; }
            case 'x': { if (force[1] > -1.0) force[1] = force[1]-0.1; break; }

            case '3': { force[2] =  1.0; break; }
            case 'e': { force[2] = -1.0; break; }
            case 'd': { if (force[2] <  1.0) force[2] = force[2]+0.1; break; }
            case 'c': { if (force[2] > -1.0) force[2] = force[2]-0.1; break; }

            case '4': { force[3] =  1.0; break; }
            case 'r': { force[3] = -1.0; break; }
            case 'f': { if (force[3] <  1.0) force[3] = force[3]+0.1; break; }
            case 'v': { if (force[3] > -1.0) force[3] = force[3]-0.1; break; }

            case '5': { force[4] =  1.0; break; }
            case 't': { force[4] = -1.0; break; }
            case 'g': { if (force[4] <  1.0) force[4] = force[4]+0.1; break; }
            case 'b': { if (force[4] > -1.0) force[4] = force[4]-0.1; break; }

            case ' ': { // Space: alle Kraefte auf Null...
               force[0]=0.0;
               force[1]=0.0;
               force[2]=0.0;
               force[3]=0.0;
               force[4]=0.0;
               break;
            }   
                
         }
         
      }   
   } while (cc != 27); // ESC
   printf("\n");
   tty_reset(STDIN_FILENO);     
}
